"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.stringifyCSSSheet = function (sheet, protocol) {
    return sheet.rules.map(function (rule) { return stringifyCSSRule(rule, protocol); }).join("\n");
};
var stringifyCSSRule = function (rule, protocol) {
    switch (rule.kind) {
        case "Style":
            return stringifyStyleRule(rule, protocol);
        case "Page":
        case "Supports":
        case "Media":
            return stringifyConditionRule(rule, protocol);
        case "FontFace":
            return stringifyFontFaceRule(rule, protocol);
    }
};
var stringifyConditionRule = function (_a, protocol) {
    var name = _a.name, condition_text = _a.condition_text, rules = _a.rules;
    return "@" + name + " " + condition_text + " {\n    " + rules.map(function (style) { return stringifyStyleRule(style, protocol); }).join("\n") + "\n  }";
};
var stringifyFontFaceRule = function (_a, protocol) {
    var style = _a.style;
    return "@font-face {\n    " + style.map(function (style) { return stringifyStyle(style, protocol); }).join("\n") + "\n  }";
};
var stringifyStyleRule = function (_a, protocol) {
    var selector_text = _a.selector_text, style = _a.style;
    return selector_text + " {\n    " + style.map(function (style) { return stringifyStyle(style, protocol); }).join("\n") + "\n  }";
};
var stringifyStyle = function (_a, protocol) {
    var name = _a.name, value = _a.value;
    return name + ":" + (value && protocol ? value.replace(/file:/, protocol) : value) + ";";
};
