"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var __1 = require("../..");
var chai_1 = require("chai");
describe(__filename + "#", function () {
    var cases = [
        [
            "{a}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "{a.b}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 0,
                            fromSpread: false,
                            properties: {
                                b: {
                                    optional: false,
                                    value: {
                                        kind: 2
                                    }
                                }
                            }
                        }
                    }
                }
            }
        ],
        [
            "{{a, b, c: [d, {e, f: b.d}]}}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    },
                    b: {
                        optional: false,
                        value: {
                            kind: 0,
                            fromSpread: false,
                            properties: {
                                d: {
                                    optional: false,
                                    value: {
                                        kind: 2
                                    }
                                }
                            }
                        }
                    },
                    d: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    },
                    e: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "{a.b} {a.c}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 0,
                            fromSpread: false,
                            properties: {
                                b: {
                                    optional: false,
                                    value: {
                                        kind: 2
                                    }
                                },
                                c: {
                                    optional: false,
                                    value: {
                                        kind: 2
                                    }
                                }
                            }
                        }
                    }
                }
            }
        ],
        [
            "<span {a}></span>",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "<span a={a}></span>",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "<span {...a}></span>",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 0,
                            fromSpread: true,
                            properties: {}
                        }
                    }
                }
            }
        ],
        [
            "{#each items as item}{/}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    items: {
                        optional: false,
                        value: {
                            kind: 1,
                            value: {
                                kind: 2
                            }
                        }
                    }
                }
            }
        ],
        [
            "{#each items as item, i}{item.name} {i}{/}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    items: {
                        optional: false,
                        value: {
                            kind: 1,
                            value: {
                                kind: 0,
                                fromSpread: false,
                                properties: {
                                    name: {
                                        optional: false,
                                        value: {
                                            kind: 2
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        ],
        [
            "{#each items as item, i}{item.a} {item.b}{/}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    items: {
                        optional: false,
                        value: {
                            kind: 1,
                            value: {
                                kind: 0,
                                fromSpread: false,
                                properties: {
                                    a: {
                                        optional: false,
                                        value: {
                                            kind: 2
                                        }
                                    },
                                    b: {
                                        optional: false,
                                        value: {
                                            kind: 2
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        ],
        [
            "\n      {#each people as person} \n        {#each person.friends as friend}\n          {friend.name}\n\n        {/}\n      {/}\n    ",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    people: {
                        optional: false,
                        value: {
                            kind: 1,
                            value: {
                                kind: 0,
                                fromSpread: false,
                                properties: {
                                    friends: {
                                        optional: false,
                                        value: {
                                            kind: 1,
                                            value: {
                                                kind: 0,
                                                fromSpread: false,
                                                properties: {
                                                    name: {
                                                        optional: false,
                                                        value: {
                                                            kind: 2
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        ],
        [
            "{#if a}{/}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "{#if a}{/else if b} {/}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    },
                    b: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "{#if true}{cc} {/}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    cc: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "<div a={<div>{a}</div>}></div>",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "<div component as=\"a\">\n        {a}\n      </div>{b}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    },
                    b: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ]
    ];
    var engine = new __1.Engine({}, function () { });
    var _loop_1 = function (source, inference) {
        it("can infer " + source, function () {
            var ast = engine.parseContent(String(source));
            chai_1.expect(__1.infer(ast)).to.eql(inference);
        });
    };
    for (var _i = 0, cases_1 = cases; _i < cases_1.length; _i++) {
        var _a = cases_1[_i], source = _a[0], inference = _a[1];
        _loop_1(source, inference);
    }
});
